# repomd_parser -- Ruby gem to parse RPM repository metadata
# Copyright (C) 2024 Ivan Kapelyukhin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

require 'zstd-ruby'

class RepomdParser::ZstdReader
  def initialize(io_object)
    @io = io_object
    @stream = Zstd::StreamingDecompress.new
    @buffer = ''
  end

  def read(len = nil, out = nil)
    @buffer << @stream.decompress(@io.read(len)) while @buffer.size < len && !@io.eof

    if @buffer.size > len
      out = @buffer[0..len]
      @buffer = @buffer[len..-1]
    else
      out = @buffer
      @buffer = ''
    end

    out
  end

  def close
    @io.close
  end
end
